@@ -28,12 +28,10 @@ enum {
 	PROP_0,
 	PROP_CURRENT_FOLDER,
 	PROP_FILENAME,
-	PROP_HTML,
 	PROP_HTML_MODE,
 	PROP_INLINE_SPELLING,
 	PROP_MAGIC_LINKS,
-	PROP_MAGIC_SMILEYS,
-	PROP_CHANGED
+	PROP_MAGIC_SMILEYS
 };
 
 enum {
@@ -57,19 +55,28 @@ editor_alignment_changed_cb (GtkhtmlEditor *editor,
 		GTK_RADIO_ACTION (ACTION (JUSTIFY_CENTER)), alignment);
 }
 
-static void
-editor_show_popup_menu (GtkhtmlEditor *editor,
-                        GdkEventButton *event,
-                        HTMLObject *object,
-			guint offset)
+static gboolean
+editor_button_press_event_cb (GtkhtmlEditor *editor,
+                              GdkEventButton *event)
 {
 	GtkHTML *html;
 	GtkWidget *menu;
+	HTMLObject *object;
 	gboolean in_selection;
+	guint offset;
 
 	html = gtkhtml_editor_get_html (editor);
 	menu = gtkhtml_editor_get_managed_widget (editor, "/context-menu");
 
+	object = html_engine_get_object_at (
+		html->engine, event->x, event->y, &offset, FALSE);
+
+	if (event->type != GDK_BUTTON_PRESS)
+		return FALSE;
+
+	if (event->button != 3)
+		return FALSE;
+
 	/* Did we right-click in a selection? */
 	in_selection =
 		html_engine_is_selection_active (html->engine) &&
@@ -77,45 +84,15 @@ editor_show_popup_menu (GtkhtmlEditor *editor,
 
 	if (!in_selection) {
 		html_engine_disable_selection (html->engine);
-		if (event != NULL)
-			html_engine_jump_at (
-				html->engine, event->x, event->y);
+		html_engine_jump_at (html->engine, event->x, event->y);
+		gtkhtml_editor_update_context (editor);
 	}
 
-	gtkhtml_editor_update_context (editor);
+	gtk_menu_popup (
+		GTK_MENU (menu), NULL, NULL, NULL, NULL,
+		event->button, event->time);
 
-	if (event != NULL)
-		gtk_menu_popup (
-			GTK_MENU (menu), NULL, NULL, NULL, NULL,
-			event->button, event->time);
-	else
-		gtk_menu_popup (
-			GTK_MENU (menu), NULL, NULL, NULL, NULL,
-			0, gtk_get_current_event_time ());
-}
-
-static gboolean
-editor_button_press_event_cb (GtkhtmlEditor *editor,
-                              GdkEventButton *event)
-{
-	GtkHTML *html;
-	HTMLObject *object;
-	guint offset;
-
-	if (event->type != GDK_BUTTON_PRESS)
-		return FALSE;
-
-	if (event->button != 3)
-		return FALSE;
-
-	html = gtkhtml_editor_get_html (editor);
-
-	object = html_engine_get_object_at (
-		html->engine, event->x, event->y, &offset, FALSE);
-
-	editor_show_popup_menu (editor, event, object, offset);
-
-	return TRUE;
+	return FALSE;
 }
 
 static void
@@ -124,8 +101,6 @@ editor_font_style_changed_cb (GtkhtmlEditor *editor,
 {
 	GtkHTMLFontStyle size;
 
-	editor->priv->ignore_style_change++;
-
 	gtk_toggle_action_set_active (
 		GTK_TOGGLE_ACTION (ACTION (BOLD)),
 		style & GTK_HTML_FONT_STYLE_BOLD);
@@ -152,8 +127,6 @@ editor_font_style_changed_cb (GtkhtmlEditor *editor,
 
 	gtk_radio_action_set_current_value (
 		GTK_RADIO_ACTION (ACTION (SIZE_PLUS_ZERO)), size);
-
-	editor->priv->ignore_style_change--;
 }
 
 static void
@@ -167,26 +140,8 @@ static void
 editor_paragraph_style_changed_cb (GtkhtmlEditor *editor,
                                    GtkHTMLParagraphStyle style)
 {
-	editor->priv->ignore_style_change++;
-
 	gtk_radio_action_set_current_value (
 		GTK_RADIO_ACTION (ACTION (STYLE_NORMAL)), style);
-
-	editor->priv->ignore_style_change--;
-}
-
-static gboolean
-editor_popup_menu_cb (GtkhtmlEditor *editor)
-{
-	GtkHTML *html;
-	HTMLObject *object;
-
-	html = gtkhtml_editor_get_html (editor);
-	object = html->engine->cursor->object;
-
-	editor_show_popup_menu (editor, NULL, object, html->engine->cursor->offset);
-
-	return TRUE;
 }
 
 static void
@@ -214,13 +169,47 @@ editor_url_requested_cb (GtkhtmlEditor *editor,
                          const gchar *url,
                          GtkHTMLStream *stream)
 {
-	g_signal_emit (editor, signals[URI_REQUESTED], 0, url, stream);
-}
+	GtkHTML *html;
+	GMappedFile *mapped_file;
+	GtkHTMLStreamStatus status;
+	gchar *filename = NULL;
+	GError *error = NULL;
 
-static void
-engine_undo_changed_cb (GtkhtmlEditor *editor, HTMLEngine *engine)
-{
-	g_object_notify (G_OBJECT (editor), "changed");
+	html = gtkhtml_editor_get_html (editor);
+
+	/* We can only handle local URLs here. */
+	if (g_ascii_strncasecmp (url, "file:/", 6) != 0) {
+		g_signal_emit (editor, signals[URI_REQUESTED], 0, url, stream);
+		return;
+	}
+
+	filename = g_filename_from_uri (url, NULL, &error);
+	if (filename == NULL)
+		goto exit;
+
+	mapped_file = g_mapped_file_new (filename, FALSE, &error);
+	if (mapped_file == NULL)
+		goto exit;
+
+	gtk_html_write (
+		html, stream,
+		g_mapped_file_get_contents (mapped_file),
+		g_mapped_file_get_length (mapped_file));
+
+	g_mapped_file_free (mapped_file);
+
+exit:
+	if (error == NULL)
+		status = GTK_HTML_STREAM_OK;
+	else {
+		status = GTK_HTML_STREAM_ERROR;
+		g_warning ("%s", error->message);
+		g_error_free (error);
+	}
+
+	gtk_html_end (html, stream, status);
+
+	g_free (filename);
 }
 
 static void
@@ -250,17 +239,12 @@ editor_method_check_word (GtkHTML *html,
                           gpointer user_data)
 {
 	GtkhtmlEditor *editor = user_data;
-	gboolean correct = FALSE;
+	gboolean correct = TRUE;
 	GList *list;
 
 	list = editor->priv->active_spell_checkers;
 
-	/* If no spell checkers are active, assume the word is correct. */
-	if (list == NULL)
-		return TRUE;
-
-	/* The word is correct if ANY active spell checker can verify it. */
-	while (list != NULL && !correct) {
+	while (list != NULL && correct) {
 		GtkhtmlSpellChecker *checker = list->data;
 
 		correct = gtkhtml_spell_checker_check_word (checker, word, -1);
@@ -348,7 +332,7 @@ editor_method_event (GtkHTML *html,
 	guint signal_id;
 
 	/* GtkHTML event arguments are either NULL or a single string. */
-	if (args != NULL && G_VALUE_HOLDS (args, G_TYPE_STRING))
+	if (G_VALUE_HOLDS (args, G_TYPE_STRING))
 		string = g_value_get_string (args);
 
 	switch (event) {
@@ -438,20 +422,27 @@ static GtkHTMLEditorAPI editor_api = {
 };
 
 static void
-editor_set_html (GtkhtmlEditor *editor,
-                 GtkHTML *html)
+editor_cut_clipboard (GtkhtmlEditor *editor)
 {
-	g_return_if_fail (editor->priv->edit_area == NULL);
+	gtk_html_command (gtkhtml_editor_get_html (editor), "cut");
+}
 
-	if (html == NULL)
-		html = (GtkHTML *) gtk_html_new ();
-	else
-		g_return_if_fail (GTK_IS_HTML (html));
+static void
+editor_copy_clipboard (GtkhtmlEditor *editor)
+{
+	gtk_html_command (gtkhtml_editor_get_html (editor), "copy");
+}
 
-	gtk_html_load_empty (html);
-	gtk_html_set_editable (html, TRUE);
+static void
+editor_paste_clipboard (GtkhtmlEditor *editor)
+{
+	gtk_html_command (gtkhtml_editor_get_html (editor), "paste");
+}
 
-	editor->priv->edit_area = g_object_ref_sink (html);
+static void
+editor_select_all (GtkhtmlEditor *editor)
+{
+	gtk_html_command (gtkhtml_editor_get_html (editor), "select-all");
 }
 
 static GObject *
@@ -459,7 +450,6 @@ editor_constructor (GType type,
                     guint n_construct_properties,
                     GObjectConstructParam *construct_properties)
 {
-	GtkHTML *html;
 	GtkhtmlEditor *editor;
 	GObject *object;
 
@@ -469,92 +459,51 @@ editor_constructor (GType type,
 
 	editor = GTKHTML_EDITOR (object);
 
-	gtkhtml_editor_private_constructed (editor);
-
-	html = gtkhtml_editor_get_html (editor);
-	gtk_html_set_editor_api (html, &editor_api, editor);
-
 	gtk_container_add (GTK_CONTAINER (object), editor->vbox);
 	gtk_window_set_default_size (GTK_WINDOW (object), 600, 440);
 
-	/* Listen for events from core and update the UI accordingly. */
-
-	g_signal_connect_swapped (
-		html, "button_press_event",
-		G_CALLBACK (editor_button_press_event_cb), editor);
-
-	g_signal_connect_swapped (
-		html, "current_paragraph_alignment_changed",
-		G_CALLBACK (editor_alignment_changed_cb), editor);
-
-	g_signal_connect_swapped (
-		html, "current_paragraph_indentation_changed",
-		G_CALLBACK (editor_indentation_changed_cb), editor);
-
-	g_signal_connect_swapped (
-		html, "current_paragraph_style_changed",
-		G_CALLBACK (editor_paragraph_style_changed_cb), editor);
-
-	g_signal_connect_swapped (
-		html, "insertion_font_style_changed",
-		G_CALLBACK (editor_font_style_changed_cb), editor);
-
-	g_signal_connect_swapped (
-		html, "popup-menu",
-		G_CALLBACK (editor_popup_menu_cb), editor);
-
-	g_signal_connect_swapped (
-		html, "url_requested",
-		G_CALLBACK (editor_url_requested_cb), editor);
-
-	g_signal_connect_swapped (
-		html->engine, "undo-changed",
-		G_CALLBACK (engine_undo_changed_cb), editor);
-
-	/* Connect property dialog widgets to actions. */
-
-	gtk_activatable_set_related_action (
-		GTK_ACTIVATABLE (WIDGET (TEXT_PROPERTIES_BOLD_BUTTON)),
-		ACTION (BOLD));
-	gtk_activatable_set_related_action (
-		GTK_ACTIVATABLE (WIDGET (REPLACE_CONFIRMATION_REPLACE_BUTTON)),
-		ACTION (CONFIRM_REPLACE));
-	gtk_activatable_set_related_action (
-		GTK_ACTIVATABLE (WIDGET (REPLACE_CONFIRMATION_REPLACE_ALL_BUTTON)),
-		ACTION (CONFIRM_REPLACE_ALL));
-	gtk_activatable_set_related_action (
-		GTK_ACTIVATABLE (WIDGET (REPLACE_CONFIRMATION_CLOSE_BUTTON)),
-		ACTION (CONFIRM_REPLACE_CANCEL));
-	gtk_activatable_set_related_action (
-		GTK_ACTIVATABLE (WIDGET (REPLACE_CONFIRMATION_NEXT_BUTTON)),
-		ACTION (CONFIRM_REPLACE_NEXT));
-	gtk_activatable_set_related_action (
-		GTK_ACTIVATABLE (WIDGET (FIND_BUTTON)),
-		ACTION (FIND));
-	gtk_activatable_set_related_action (
-		GTK_ACTIVATABLE (WIDGET (REPLACE_BUTTON)),
-		ACTION (FIND_AND_REPLACE));
-	gtk_activatable_set_related_action (
-		GTK_ACTIVATABLE (WIDGET (TEXT_PROPERTIES_ITALIC_BUTTON)),
-		ACTION (ITALIC));
-	gtk_activatable_set_related_action (
-		GTK_ACTIVATABLE (WIDGET (PARAGRAPH_PROPERTIES_CENTER_BUTTON)),
-		ACTION (JUSTIFY_CENTER));
-	gtk_activatable_set_related_action (
-		GTK_ACTIVATABLE (WIDGET (PARAGRAPH_PROPERTIES_LEFT_BUTTON)),
-		ACTION (JUSTIFY_LEFT));
-	gtk_activatable_set_related_action (
-		GTK_ACTIVATABLE (WIDGET (PARAGRAPH_PROPERTIES_RIGHT_BUTTON)),
-		ACTION (JUSTIFY_RIGHT));
-	gtk_activatable_set_related_action (
-		GTK_ACTIVATABLE (WIDGET (TEXT_PROPERTIES_STRIKETHROUGH_BUTTON)),
-		ACTION (STRIKETHROUGH));
-	gtk_activatable_set_related_action (
-		GTK_ACTIVATABLE (WIDGET (LINK_PROPERTIES_TEST_BUTTON)),
-		ACTION (TEST_URL));
-	gtk_activatable_set_related_action (
-		GTK_ACTIVATABLE (WIDGET (TEXT_PROPERTIES_UNDERLINE_BUTTON)),
-		ACTION (UNDERLINE));
+	gtk_action_connect_proxy (
+		ACTION (BOLD),
+		WIDGET (TEXT_PROPERTIES_BOLD_BUTTON));
+	gtk_action_connect_proxy (
+		ACTION (CONFIRM_REPLACE),
+		WIDGET (REPLACE_CONFIRMATION_REPLACE_BUTTON));
+	gtk_action_connect_proxy (
+		ACTION (CONFIRM_REPLACE_ALL),
+		WIDGET (REPLACE_CONFIRMATION_REPLACE_ALL_BUTTON));
+	gtk_action_connect_proxy (
+		ACTION (CONFIRM_REPLACE_CANCEL),
+		WIDGET (REPLACE_CONFIRMATION_CLOSE_BUTTON));
+	gtk_action_connect_proxy (
+		ACTION (CONFIRM_REPLACE_NEXT),
+		WIDGET (REPLACE_CONFIRMATION_NEXT_BUTTON));
+	gtk_action_connect_proxy (
+		ACTION (FIND),
+		WIDGET (FIND_BUTTON));
+	gtk_action_connect_proxy (
+		ACTION (FIND_AND_REPLACE),
+		WIDGET (REPLACE_BUTTON));
+	gtk_action_connect_proxy (
+		ACTION (ITALIC),
+		WIDGET (TEXT_PROPERTIES_ITALIC_BUTTON));
+	gtk_action_connect_proxy (
+		ACTION (JUSTIFY_CENTER),
+		WIDGET (PARAGRAPH_PROPERTIES_CENTER_BUTTON));
+	gtk_action_connect_proxy (
+		ACTION (JUSTIFY_LEFT),
+		WIDGET (PARAGRAPH_PROPERTIES_LEFT_BUTTON));
+	gtk_action_connect_proxy (
+		ACTION (JUSTIFY_RIGHT),
+		WIDGET (PARAGRAPH_PROPERTIES_RIGHT_BUTTON));
+	gtk_action_connect_proxy (
+		ACTION (STRIKETHROUGH),
+		WIDGET (TEXT_PROPERTIES_STRIKETHROUGH_BUTTON));
+	gtk_action_connect_proxy (
+		ACTION (TEST_URL),
+		WIDGET (LINK_PROPERTIES_TEST_BUTTON));
+	gtk_action_connect_proxy (
+		ACTION (UNDERLINE),
+		WIDGET (TEXT_PROPERTIES_UNDERLINE_BUTTON));
 
 	gtk_window_set_transient_for (
 		GTK_WINDOW (WIDGET (CELL_PROPERTIES_WINDOW)),
@@ -626,12 +575,6 @@ editor_set_property (GObject *object,
 				g_value_get_string (value));
 			return;
 
-		case PROP_HTML:
-			editor_set_html (
-				GTKHTML_EDITOR (object),
-				g_value_get_object (value));
-			return;
-
 		case PROP_HTML_MODE:
 			gtkhtml_editor_set_html_mode (
 				GTKHTML_EDITOR (object),
@@ -655,11 +598,6 @@ editor_set_property (GObject *object,
 				GTKHTML_EDITOR (object),
 				g_value_get_boolean (value));
 			return;
-		case PROP_CHANGED:
-			gtkhtml_editor_set_changed (
-				GTKHTML_EDITOR (object),
-				g_value_get_boolean (value));
-			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -684,12 +622,6 @@ editor_get_property (GObject *object,
 				GTKHTML_EDITOR (object)));
 			return;
 
-		case PROP_HTML:
-			g_value_set_object (
-				value, gtkhtml_editor_get_html (
-				GTKHTML_EDITOR (object)));
-			return;
-
 		case PROP_HTML_MODE:
 			g_value_set_boolean (
 				value, gtkhtml_editor_get_html_mode (
@@ -713,11 +645,6 @@ editor_get_property (GObject *object,
 				value, gtkhtml_editor_get_magic_smileys (
 				GTKHTML_EDITOR (object)));
 			return;
-		case PROP_CHANGED:
-			g_value_set_boolean (
-				value, gtkhtml_editor_get_changed (
-				GTKHTML_EDITOR (object)));
-			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -747,52 +674,10 @@ editor_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static gboolean
-editor_key_press_event (GtkWidget *widget,
-                        GdkEventKey *event)
-{
-#ifdef HAVE_XFREE
-	GtkhtmlEditor *editor = GTKHTML_EDITOR (widget);
-
-	if (event->keyval == XF86XK_Spell) {
-		gtk_action_activate (ACTION (SPELL_CHECK));
-		return TRUE;
-	}
-#endif
-
-	/* Chain up to parent's key_press_event() method. */
-	return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
-}
-
-static void
-editor_cut_clipboard (GtkhtmlEditor *editor)
-{
-	gtk_html_command (gtkhtml_editor_get_html (editor), "cut");
-}
-
-static void
-editor_copy_clipboard (GtkhtmlEditor *editor)
-{
-	gtk_html_command (gtkhtml_editor_get_html (editor), "copy");
-}
-
-static void
-editor_paste_clipboard (GtkhtmlEditor *editor)
-{
-	gtk_html_command (gtkhtml_editor_get_html (editor), "paste");
-}
-
-static void
-editor_select_all (GtkhtmlEditor *editor)
-{
-	gtk_html_command (gtkhtml_editor_get_html (editor), "select-all");
-}
-
 static void
 editor_class_init (GtkhtmlEditorClass *class)
 {
 	GObjectClass *object_class;
-	GtkWidgetClass *widget_class;
 
 	parent_class = g_type_class_peek_parent (class);
 	g_type_class_add_private (class, sizeof (GtkhtmlEditorPrivate));
@@ -804,9 +689,6 @@ editor_class_init (GtkhtmlEditorClass *class)
 	object_class->dispose = editor_dispose;
 	object_class->finalize = editor_finalize;
 
-	widget_class = GTK_WIDGET_CLASS (class);
-	widget_class->key_press_event = editor_key_press_event;
-
 	class->cut_clipboard = editor_cut_clipboard;
 	class->copy_clipboard = editor_copy_clipboard;
 	class->paste_clipboard = editor_paste_clipboard;
@@ -817,86 +699,65 @@ editor_class_init (GtkhtmlEditorClass *class)
 		PROP_CURRENT_FOLDER,
 		g_param_spec_string (
 			"current-folder",
-			"Current Folder",
-			"The initial folder for file chooser dialogs",
+			_("Current Folder"),
+			_("The initial folder for file chooser dialogs"),
 			g_get_home_dir (),
-			G_PARAM_READWRITE |
-			G_PARAM_CONSTRUCT));
+			G_PARAM_CONSTRUCT |
+			G_PARAM_READWRITE));
 
 	g_object_class_install_property (
 		object_class,
 		PROP_FILENAME,
 		g_param_spec_string (
 			"filename",
-			"Filename",
-			"The filename to use when saving",
+			_("Filename"),
+			_("The filename to use when saving"),
 			NULL,
-			G_PARAM_READWRITE |
-			G_PARAM_CONSTRUCT));
-
-	g_object_class_install_property (
-		object_class,
-		PROP_HTML,
-		g_param_spec_object (
-			"html",
-			"HTML Editing Widget",
-			"The main HTML editing widget",
-			GTK_TYPE_HTML,
-			G_PARAM_READWRITE |
-			G_PARAM_CONSTRUCT_ONLY));
+			G_PARAM_CONSTRUCT |
+			G_PARAM_READWRITE));
 
 	g_object_class_install_property (
 		object_class,
 		PROP_HTML_MODE,
 		g_param_spec_boolean (
 			"html-mode",
-			"HTML Mode",
-			"Edit HTML or plain text",
+			_("HTML Mode"),
+			_("Edit HTML or plain text"),
 			TRUE,
-			G_PARAM_READWRITE |
-			G_PARAM_CONSTRUCT));
+			G_PARAM_CONSTRUCT |
+			G_PARAM_READWRITE));
 
 	g_object_class_install_property (
 		object_class,
 		PROP_INLINE_SPELLING,
 		g_param_spec_boolean (
 			"inline-spelling",
-			"Inline Spelling",
-			"Check your spelling as you type",
+			_("Inline Spelling"),
+			_("Check your spelling as you type"),
 			TRUE,
-			G_PARAM_READWRITE |
-			G_PARAM_CONSTRUCT));
+			G_PARAM_CONSTRUCT |
+			G_PARAM_READWRITE));
 
 	g_object_class_install_property (
 		object_class,
 		PROP_MAGIC_LINKS,
 		g_param_spec_boolean (
 			"magic-links",
-			"Magic Links",
-			"Make URIs clickable as you type",
+			_("Magic Links"),
+			_("Make URIs clickable as you type"),
 			TRUE,
-			G_PARAM_READWRITE |
-			G_PARAM_CONSTRUCT));
+			G_PARAM_CONSTRUCT |
+			G_PARAM_READWRITE));
 
 	g_object_class_install_property (
 		object_class,
 		PROP_MAGIC_SMILEYS,
 		g_param_spec_boolean (
 			"magic-smileys",
-			"Magic Smileys",
-			"Convert emoticons to images as you type",
+			_("Magic Smileys"),
+			_("Convert emoticons to images as you type"),
 			TRUE,
-			G_PARAM_READWRITE |
-			G_PARAM_CONSTRUCT));
-
-	g_object_class_install_property (
-		object_class,
-		PROP_CHANGED,
-		g_param_spec_boolean (
-			"changed",
-			_("Changed property"),
-			_("Whether editor changed"),
-			FALSE,
+			G_PARAM_CONSTRUCT |
 			G_PARAM_READWRITE));
 
 	signals[COMMAND_AFTER] = g_signal_new (
@@ -963,14 +824,48 @@ editor_class_init (GtkhtmlEditorClass *class)
 static void
 editor_init (GtkhtmlEditor *editor)
 {
+	GtkHTML *html;
+
 	editor->priv = GTKHTML_EDITOR_GET_PRIVATE (editor);
 	editor->vbox = g_object_ref_sink (gtk_vbox_new (FALSE, 0));
 	gtk_widget_show (editor->vbox);
 
-	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-
 	gtkhtml_editor_private_init (editor);
+
+	html = gtkhtml_editor_get_html (editor);
+	gtk_html_set_editor_api (html, &editor_api, editor);
+
+	/* Listen for events from core and update the UI accordingly. */
+
+	g_signal_connect_swapped (
+		gtkhtml_editor_get_html (editor),
+		"button_press_event",
+		G_CALLBACK (editor_button_press_event_cb), editor);
+
+	g_signal_connect_swapped (
+		gtkhtml_editor_get_html (editor),
+		"current_paragraph_alignment_changed",
+		G_CALLBACK (editor_alignment_changed_cb), editor);
+
+	g_signal_connect_swapped (
+		gtkhtml_editor_get_html (editor),
+		"current_paragraph_indentation_changed",
+		G_CALLBACK (editor_indentation_changed_cb), editor);
+
+	g_signal_connect_swapped (
+		gtkhtml_editor_get_html (editor),
+		"current_paragraph_style_changed",
+		G_CALLBACK (editor_paragraph_style_changed_cb), editor);
+
+	g_signal_connect_swapped (
+		gtkhtml_editor_get_html (editor),
+		"insertion_font_style_changed",
+		G_CALLBACK (editor_font_style_changed_cb), editor);
+
+	g_signal_connect_swapped (
+		gtkhtml_editor_get_html (editor),
+		"url_requested",
+		G_CALLBACK (editor_url_requested_cb), editor);
 }
 
 GType
@@ -1013,14 +908,6 @@ gtkhtml_editor_get_html (GtkhtmlEditor *editor)
 	return GTK_HTML (editor->priv->edit_area);
 }
 
-GtkBuilder *
-gtkhtml_editor_get_builder (GtkhtmlEditor *editor)
-{
-	g_return_val_if_fail (GTKHTML_IS_EDITOR (editor), NULL);
-
-	return editor->priv->builder;
-}
-
 GtkUIManager *
 gtkhtml_editor_get_ui_manager (GtkhtmlEditor *editor)
 {
@@ -1033,16 +920,13 @@ GtkAction *
 gtkhtml_editor_get_action (GtkhtmlEditor *editor,
                            const gchar *action_name)
 {
-	GtkUIManager *manager;
 	GtkAction *action = NULL;
 	GList *iter;
 
 	g_return_val_if_fail (GTKHTML_IS_EDITOR (editor), NULL);
 	g_return_val_if_fail (action_name != NULL, NULL);
 
-	manager = gtkhtml_editor_get_ui_manager (editor);
-	iter = gtk_ui_manager_get_action_groups (manager);
-
+	iter = gtk_ui_manager_get_action_groups (editor->priv->manager);
 	while (iter != NULL && action == NULL) {
 		GtkActionGroup *action_group = iter->data;
 
@@ -1050,8 +934,7 @@ gtkhtml_editor_get_action (GtkhtmlEditor *editor,
 			action_group, action_name);
 		iter = g_list_next (iter);
 	}
-
-	g_return_val_if_fail (action != NULL, NULL);
+	g_assert (action != NULL);
 
 	return action;
 }
@@ -1060,15 +943,12 @@ GtkActionGroup *
 gtkhtml_editor_get_action_group (GtkhtmlEditor *editor,
                                  const gchar *group_name)
 {
-	GtkUIManager *manager;
 	GList *iter;
 
 	g_return_val_if_fail (GTKHTML_IS_EDITOR (editor), NULL);
 	g_return_val_if_fail (group_name != NULL, NULL);
 
-	manager = gtkhtml_editor_get_ui_manager (editor);
-	iter = gtk_ui_manager_get_action_groups (manager);
-
+	iter = gtk_ui_manager_get_action_groups (editor->priv->manager);
 	while (iter != NULL) {
 		GtkActionGroup *action_group = iter->data;
 		const gchar *name;
@@ -1087,17 +967,15 @@ GtkWidget *
 gtkhtml_editor_get_widget (GtkhtmlEditor *editor,
                            const gchar *widget_name)
 {
-	GtkBuilder *builder;
-	GObject *object;
+	GtkWidget *widget;
 
 	g_return_val_if_fail (GTKHTML_IS_EDITOR (editor), NULL);
 	g_return_val_if_fail (widget_name != NULL, NULL);
 
-	builder = gtkhtml_editor_get_builder (editor);
-	object = gtk_builder_get_object (builder, widget_name);
-	g_return_val_if_fail (GTK_IS_WIDGET (object), NULL);
+	widget = glade_xml_get_widget (editor->priv->glade_xml, widget_name);
+	g_assert (widget != NULL);
 
-	return GTK_WIDGET (object);
+	return widget;
 }
 
 GtkWidget *
@@ -1112,8 +990,7 @@ gtkhtml_editor_get_managed_widget (GtkhtmlEditor *editor,
 
 	manager = gtkhtml_editor_get_ui_manager (editor);
 	widget = gtk_ui_manager_get_widget (manager, widget_path);
-
-	g_return_val_if_fail (widget != NULL, NULL);
+	g_assert (widget != NULL);
 
 	return widget;
 }
@@ -1146,22 +1023,12 @@ gtkhtml_editor_set_changed (GtkhtmlEditor *editor,
 
 	g_return_if_fail (GTKHTML_IS_EDITOR (editor));
 
-	if (!changed) {
-		html = gtkhtml_editor_get_html (editor);
+	html = gtkhtml_editor_get_html (editor);
 
-		/* XXX The NULL check was added to deal with a race in
-		 *     Evolution when saving a message to a remote Drafts
-		 *     folder (which is asynchronous) and then closing the
-		 *     editor (which is immediate).  The GtkHTML object
-		 *     may already be disposed by the time the Save Draft
-		 *     callback gets here.  See bug #553995 for details. */
-		if (html)
-			html_engine_saved (html->engine);
-	}
+	if (!changed)
+		html_engine_saved (html->engine);
 
 	editor->priv->changed = changed;
-
-	g_object_notify (G_OBJECT (editor), "changed");
 }
 
 const gchar *
@@ -1210,31 +1077,27 @@ gtkhtml_editor_set_filename (GtkhtmlEditor *editor,
 gboolean
 gtkhtml_editor_get_html_mode (GtkhtmlEditor *editor)
 {
-	GtkRadioAction *action;
-	EditorMode mode;
+	GtkToggleAction *action;
 
 	g_return_val_if_fail (GTKHTML_IS_EDITOR (editor), FALSE);
 
-	action = GTK_RADIO_ACTION (ACTION (MODE_HTML));
-	mode = gtk_radio_action_get_current_value (action);
+	action = GTK_TOGGLE_ACTION (ACTION (HTML_MODE));
 
-	return (mode == EDITOR_MODE_HTML);
+	return gtk_toggle_action_get_active (action);
 }
 
 void
 gtkhtml_editor_set_html_mode (GtkhtmlEditor *editor,
                               gboolean html_mode)
 {
-	GtkRadioAction *action;
-	EditorMode mode;
+	GtkToggleAction *action;
 
 	g_return_if_fail (GTKHTML_IS_EDITOR (editor));
 
-	action = GTK_RADIO_ACTION (ACTION (MODE_HTML));
-	mode = html_mode ? EDITOR_MODE_HTML : EDITOR_MODE_TEXT;
-	gtk_radio_action_set_current_value (action, mode);
+	action = GTK_TOGGLE_ACTION (ACTION (HTML_MODE));
+	gtk_toggle_action_set_active (action, html_mode);
 
-	/* We emit "notify::html-mode" in the action handler. */
+	g_object_notify (G_OBJECT (editor), "html-mode");
 }
 
 gboolean
@@ -1385,7 +1248,7 @@ gtkhtml_editor_file_chooser_dialog_run (GtkhtmlEditor *editor,
 	g_return_val_if_fail (GTKHTML_IS_EDITOR (editor), response);
 	g_return_val_if_fail (GTK_IS_FILE_CHOOSER_DIALOG (dialog), response);
 
-	gtk_file_chooser_set_current_folder_uri (
+	gtk_file_chooser_set_current_folder (
 		GTK_FILE_CHOOSER (dialog),
 		gtkhtml_editor_get_current_folder (editor));
 
@@ -1400,7 +1263,7 @@ gtkhtml_editor_file_chooser_dialog_run (GtkhtmlEditor *editor,
 	if (save_folder) {
 		gchar *folder;
 
-		folder = gtk_file_chooser_get_current_folder_uri (
+		folder = gtk_file_chooser_get_current_folder (
 			GTK_FILE_CHOOSER (dialog));
 		gtkhtml_editor_set_current_folder (editor, folder);
 		g_free (folder);
@@ -1662,38 +1525,6 @@ gtkhtml_editor_insert_html (GtkhtmlEditor *editor,
 	gtk_html_insert_html (html, html_text);
 }
 
-void
-gtkhtml_editor_insert_image (GtkhtmlEditor *editor,
-                             const gchar *image_uri)
-{
-	GtkHTML *html;
-	HTMLObject *image;
-
-	g_return_if_fail (GTKHTML_IS_EDITOR (editor));
-	g_return_if_fail (image_uri != NULL);
-
-	html = gtkhtml_editor_get_html (editor);
-
-	image = html_image_new (
-		html_engine_get_image_factory (html->engine), image_uri,
-		NULL, NULL, 0, 0, 0, 0, 0, NULL, HTML_VALIGN_NONE, FALSE);
-
-	html_engine_paste_object (html->engine, image, 1);
-}
-
-void
-gtkhtml_editor_insert_text (GtkhtmlEditor *editor,
-                            const gchar *plain_text)
-{
-	GtkHTML *html;
-
-	g_return_if_fail (GTKHTML_IS_EDITOR (editor));
-
-	html = gtkhtml_editor_get_html (editor);
-
-	html_engine_paste_text (html->engine, plain_text, -1);
-}
-
 gboolean
 gtkhtml_editor_search_by_data (GtkhtmlEditor *editor,
                                glong level,
@@ -1780,7 +1611,7 @@ gtkhtml_editor_undo_end (GtkhtmlEditor *editor)
 
 	html = gtkhtml_editor_get_html (editor);
 
-	html_undo_level_end (html->engine->undo, html->engine);
+	html_undo_level_end (html->engine->undo);
 }
 
 gboolean
